# pylama: max_line_length = 100
from PyQt5.QtCore import QObject, pyqtSignal
import math

BACKLIGHT_DIR_NAME = '/run/backlight/'


class Brightness(QObject):
    """Class to operate hardware brightness."""

    changed = pyqtSignal(int)

    def __init__(self, max_light=100):
        """__init__.

        Args:
            max_light (int, optional): set maximum value for logical light
        """
        super().__init__()
        self.max_light = max_light
        self.__light = max_light
        self.__max_brightness = max_light
        self.__coefficient = 420
        try:
            with open(BACKLIGHT_DIR_NAME + 'max_brightness', 'r') as file:
                self.__max_brightness = int(file.read()) // self.__coefficient
        except IOError:
            print('I/O error on file:' + BACKLIGHT_DIR_NAME + 'max_brightness')
        self.R = ((self.max_light) * math.log10(2))/(math.log10(self.__max_brightness));

    @property
    def light(self):
        """Get current brightness value."""
        return self.__light

    @light.setter
    def light(self, value):
        """Set brightness value.

        Converts value in 1..max_light interval to value in 1..max_brightness
        interval, and try to set it in hardware filesystem

        Args:
            value(int or str)
        """
        __old_light = self.__light
        self.__light = min(self.max_light, max(1, int(value)))

        if self.__light != __old_light:
            __new_brightness = \
                min(self.__max_brightness,
                    max(0.1, pow (2, (self.__light) / self.R) - 1))
            try:
                with open(BACKLIGHT_DIR_NAME + 'brightness', 'w') as file:
                    file.write('%d\n' % int(__new_brightness * self.__coefficient))
            except IOError:
                print('I/O error on file:' + BACKLIGHT_DIR_NAME + 'brightness')
            self.changed.emit(self.__light)

    def set(self, value):
        """Set brightness value.

        Args:
            value(int or str)
        """
        self.light = value

    def change(self, relative_value):
        """Modify brightness value.

        Change light value by specified number

        Args:
            relative_value(int)
        """
        self.light += relative_value
